@@ -1,9 +1,45 @@
 /*****************************************************************************
- * packager - A library for creating Excel XLSX packager files.
+ * packager - A library for assembling xml files into an Excel XLSX file.
  *
- * Used in conjunction with the libxlsxwriter library.
+ * A class for writing the Excel XLSX Packager file.
  *
- * Copyright 2014-2021, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
+ * This module is used in conjunction with libxlsxwriter to create an
+ * Excel XLSX container file.
+ *
+ * From Wikipedia: The Open Packaging Conventions (OPC) is a
+ * container-file technology initially created by Microsoft to store
+ * a combination of XML and non-XML files that together form a single
+ * entity such as an Open XML Paper Specification (OpenXPS)
+ * document. http://en.wikipedia.org/wiki/Open_Packaging_Conventions.
+ *
+ * At its simplest an Excel XLSX file contains the following elements::
+ *
+ *      ____ [Content_Types].xml
+ *     |
+ *     |____ docProps
+ *     | |____ app.xml
+ *     | |____ core.xml
+ *     |
+ *     |____ xl
+ *     | |____ workbook.xml
+ *     | |____ worksheets
+ *     | | |____ sheet1.xml
+ *     | |
+ *     | |____ styles.xml
+ *     | |
+ *     | |____ theme
+ *     | | |____ theme1.xml
+ *     | |
+ *     | |_____rels
+ *     | |____ workbook.xml.rels
+ *     |
+ *     |_____rels
+ *       |____ .rels
+ *
+ * The Packager class coordinates the classes that represent the
+ * elements of the package and writes them into the XLSX file.
+ *
+ * Copyright 2014-2022, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
  *
  */
 
@@ -16,9 +52,13 @@
 STATIC lxw_error _add_file_to_zip(lxw_packager *self, FILE * file,
                                   const char *filename);
 
-STATIC lxw_error _add_buffer_to_zip(lxw_packager *self, unsigned char *buffer,
+STATIC lxw_error _add_buffer_to_zip(lxw_packager *self, char *buffer,
                                     size_t buffer_size, const char *filename);
 
+STATIC lxw_error _add_to_zip(lxw_packager *self, FILE * file,
+                             char **buffer, size_t *buffer_size,
+                             const char *filename);
+
 STATIC lxw_error _write_vml_drawing_rels_file(lxw_packager *self,
                                               lxw_worksheet *worksheet,
                                               uint32_t index);
@@ -77,21 +117,75 @@ _open_zipfile_win32(const char *filename)
 
 #endif
 
+STATIC voidpf ZCALLBACK
+_fopen_memstream(voidpf opaque, const char *filename, int mode)
+{
+    lxw_packager *packager = (lxw_packager *) opaque;
+    (void) filename;
+    (void) mode;
+    return lxw_get_filehandle(&packager->output_buffer,
+                              &packager->output_buffer_size,
+                              packager->tmpdir);
+}
+
+STATIC int ZCALLBACK
+_fclose_memstream(voidpf opaque, voidpf stream)
+{
+    lxw_packager *packager = (lxw_packager *) opaque;
+    FILE *file = (FILE *) stream;
+    long size;
+
+    /* Ensure memstream buffer is updated */
+    if (fflush(file))
+        goto mem_error;
+
+    /* If the memstream is backed by a temporary file, no buffer is created,
+       so create it manually. */
+    if (!packager->output_buffer) {
+        if (fseek(file, 0L, SEEK_END))
+            goto mem_error;
+
+        size = ftell(file);
+        if (size == -1)
+            goto mem_error;
+
+        packager->output_buffer = malloc(size);
+        GOTO_LABEL_ON_MEM_ERROR(packager->output_buffer, mem_error);
+
+        rewind(file);
+        if (fread(packager->output_buffer, size, 1, file) < 1)
+            goto mem_error;
+
+        packager->output_buffer_size = size;
+    }
+
+    return fclose(file);
+
+mem_error:
+    fclose(file);
+    return EOF;
+}
+
 /*
  * Create a new packager object.
  */
 lxw_packager *
 lxw_packager_new(const char *filename, char *tmpdir, uint8_t use_zip64)
 {
+    zlib_filefunc_def filefunc;
     lxw_packager *packager = calloc(1, sizeof(lxw_packager));
     GOTO_LABEL_ON_MEM_ERROR(packager, mem_error);
 
     packager->buffer = calloc(1, LXW_ZIP_BUFFER_SIZE);
     GOTO_LABEL_ON_MEM_ERROR(packager->buffer, mem_error);
 
-    packager->filename = lxw_strdup(filename);
+    packager->filename = NULL;
     packager->tmpdir = tmpdir;
-    GOTO_LABEL_ON_MEM_ERROR(packager->filename, mem_error);
+
+    if (filename) {
+        packager->filename = lxw_strdup(filename);
+        GOTO_LABEL_ON_MEM_ERROR(packager->filename, mem_error);
+    }
 
     packager->buffer_size = LXW_ZIP_BUFFER_SIZE;
     packager->use_zip64 = use_zip64;
@@ -107,12 +201,24 @@ lxw_packager_new(const char *filename, char *tmpdir, uint8_t use_zip64)
     packager->zipfile_info.internal_fa = 0;
     packager->zipfile_info.external_fa = 0;
 
+    packager->output_buffer = NULL;
+    packager->output_buffer_size = 0;
+
     /* Create a zip container for the xlsx file. */
+    if (packager->filename) {
 #ifdef _WIN32
-    packager->zipfile = _open_zipfile_win32(packager->filename);
+        packager->zipfile = _open_zipfile_win32(packager->filename);
 #else
-    packager->zipfile = zipOpen(packager->filename, 0);
+        packager->zipfile = zipOpen(packager->filename, 0);
 #endif
+    }
+    else {
+        fill_fopen_filefunc(&filefunc);
+        filefunc.opaque = packager;
+        filefunc.zopen_file = _fopen_memstream;
+        filefunc.zclose_file = _fclose_memstream;
+        packager->zipfile = zipOpen2(packager->filename, 0, NULL, &filefunc);
+    }
 
     if (packager->zipfile == NULL)
         goto mem_error;
@@ -152,14 +258,18 @@ _write_workbook_file(lxw_packager *self)
     lxw_workbook *workbook = self->workbook;
     lxw_error err;
 
-    workbook->file = lxw_tmpfile(self->tmpdir);
+    char *buffer = NULL;
+    size_t buffer_size = 0;
+    workbook->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!workbook->file)
         return LXW_ERROR_CREATING_TMPFILE;
 
     lxw_workbook_assemble_xml_file(workbook);
 
-    err = _add_file_to_zip(self, workbook->file, "xl/workbook.xml");
+    err = _add_to_zip(self, workbook->file, &buffer, &buffer_size,
+                      "xl/workbook.xml");
     fclose(workbook->file);
+    free(buffer);
     RETURN_ON_ERROR(err);
 
     return LXW_NO_ERROR;
@@ -175,6 +285,8 @@ _write_worksheet_files(lxw_packager *self)
     lxw_sheet *sheet;
     lxw_worksheet *worksheet;
     char sheetname[LXW_FILENAME_LENGTH] = { 0 };
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t index = 1;
     lxw_error err;
 
@@ -190,14 +302,17 @@ _write_worksheet_files(lxw_packager *self)
         if (worksheet->optimize_row)
             lxw_worksheet_write_single_row(worksheet);
 
-        worksheet->file = lxw_tmpfile(self->tmpdir);
+        worksheet->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                             self->tmpdir);
         if (!worksheet->file)
             return LXW_ERROR_CREATING_TMPFILE;
 
         lxw_worksheet_assemble_xml_file(worksheet);
 
-        err = _add_file_to_zip(self, worksheet->file, sheetname);
+        err = _add_to_zip(self, worksheet->file, &buffer, &buffer_size,
+                          sheetname);
         fclose(worksheet->file);
+        free(buffer);
         RETURN_ON_ERROR(err);
     }
 
@@ -214,6 +329,8 @@ _write_chartsheet_files(lxw_packager *self)
     lxw_sheet *sheet;
     lxw_chartsheet *chartsheet;
     char sheetname[LXW_FILENAME_LENGTH] = { 0 };
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t index = 1;
     lxw_error err;
 
@@ -226,14 +343,17 @@ _write_chartsheet_files(lxw_packager *self)
         lxw_snprintf(sheetname, LXW_FILENAME_LENGTH,
                      "xl/chartsheets/sheet%d.xml", index++);
 
-        chartsheet->file = lxw_tmpfile(self->tmpdir);
+        chartsheet->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                              self->tmpdir);
         if (!chartsheet->file)
             return LXW_ERROR_CREATING_TMPFILE;
 
         lxw_chartsheet_assemble_xml_file(chartsheet);
 
-        err = _add_file_to_zip(self, chartsheet->file, sheetname);
+        err = _add_to_zip(self, chartsheet->file, &buffer, &buffer_size,
+                          sheetname);
         fclose(chartsheet->file);
+        free(buffer);
         RETURN_ON_ERROR(err);
     }
 
@@ -339,6 +459,8 @@ _write_chart_files(lxw_packager *self)
     lxw_workbook *workbook = self->workbook;
     lxw_chart *chart;
     char sheetname[LXW_FILENAME_LENGTH] = { 0 };
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t index = 1;
     lxw_error err;
 
@@ -347,14 +469,16 @@ _write_chart_files(lxw_packager *self)
         lxw_snprintf(sheetname, LXW_FILENAME_LENGTH,
                      "xl/charts/chart%d.xml", index++);
 
-        chart->file = lxw_tmpfile(self->tmpdir);
+        chart->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
         if (!chart->file)
             return LXW_ERROR_CREATING_TMPFILE;
 
         lxw_chart_assemble_xml_file(chart);
 
-        err = _add_file_to_zip(self, chart->file, sheetname);
+        err = _add_to_zip(self, chart->file, &buffer, &buffer_size,
+                          sheetname);
         fclose(chart->file);
+        free(buffer);
         RETURN_ON_ERROR(err);
     }
 
@@ -389,6 +513,8 @@ _write_drawing_files(lxw_packager *self)
     lxw_worksheet *worksheet;
     lxw_drawing *drawing;
     char filename[LXW_FILENAME_LENGTH] = { 0 };
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t index = 1;
     lxw_error err;
 
@@ -404,14 +530,17 @@ _write_drawing_files(lxw_packager *self)
             lxw_snprintf(filename, LXW_FILENAME_LENGTH,
                          "xl/drawings/drawing%d.xml", index++);
 
-            drawing->file = lxw_tmpfile(self->tmpdir);
+            drawing->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                               self->tmpdir);
             if (!drawing->file)
                 return LXW_ERROR_CREATING_TMPFILE;
 
             lxw_drawing_assemble_xml_file(drawing);
 
-            err = _add_file_to_zip(self, drawing->file, filename);
+            err = _add_to_zip(self, drawing->file, &buffer, &buffer_size,
+                              filename);
             fclose(drawing->file);
+            free(buffer);
             RETURN_ON_ERROR(err);
         }
     }
@@ -460,6 +589,8 @@ _write_table_files(lxw_packager *self)
     lxw_error err;
 
     char filename[LXW_FILENAME_LENGTH] = { 0 };
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t index = 1;
 
     STAILQ_FOREACH(sheet, workbook->sheets, list_pointers) {
@@ -482,7 +613,8 @@ _write_table_files(lxw_packager *self)
                 RETURN_ON_ERROR(err);
             }
 
-            table->file = lxw_tmpfile(self->tmpdir);
+            table->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                             self->tmpdir);
             if (!table->file) {
                 lxw_table_free(table);
                 return LXW_ERROR_CREATING_TMPFILE;
@@ -492,8 +624,10 @@ _write_table_files(lxw_packager *self)
 
             lxw_table_assemble_xml_file(table);
 
-            err = _add_file_to_zip(self, table->file, filename);
+            err = _add_to_zip(self, table->file, &buffer, &buffer_size,
+                              filename);
             fclose(table->file);
+            free(buffer);
             lxw_table_free(table);
             RETURN_ON_ERROR(err);
         }
@@ -536,6 +670,8 @@ _write_vml_files(lxw_packager *self)
     lxw_worksheet *worksheet;
     lxw_vml *vml;
     char filename[LXW_FILENAME_LENGTH] = { 0 };
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t index = 1;
     lxw_error err;
 
@@ -557,7 +693,8 @@ _write_vml_files(lxw_packager *self)
             lxw_snprintf(filename, LXW_FILENAME_LENGTH,
                          "xl/drawings/vmlDrawing%d.vml", index++);
 
-            vml->file = lxw_tmpfile(self->tmpdir);
+            vml->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                           self->tmpdir);
             if (!vml->file) {
                 lxw_vml_free(vml);
                 return LXW_ERROR_CREATING_TMPFILE;
@@ -573,15 +710,18 @@ _write_vml_files(lxw_packager *self)
             }
             else {
                 fclose(vml->file);
+                free(buffer);
                 lxw_vml_free(vml);
                 return LXW_ERROR_MEMORY_MALLOC_FAILED;
             }
 
             lxw_vml_assemble_xml_file(vml);
 
-            err = _add_file_to_zip(self, vml->file, filename);
+            err = _add_to_zip(self, vml->file, &buffer, &buffer_size,
+                              filename);
 
             fclose(vml->file);
+            free(buffer);
             lxw_vml_free(vml);
 
             RETURN_ON_ERROR(err);
@@ -599,7 +739,8 @@ _write_vml_files(lxw_packager *self)
             lxw_snprintf(filename, LXW_FILENAME_LENGTH,
                          "xl/drawings/vmlDrawing%d.vml", index++);
 
-            vml->file = lxw_tmpfile(self->tmpdir);
+            vml->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                           self->tmpdir);
             if (!vml->file) {
                 lxw_vml_free(vml);
                 return LXW_ERROR_CREATING_TMPFILE;
@@ -613,15 +754,18 @@ _write_vml_files(lxw_packager *self)
             }
             else {
                 fclose(vml->file);
+                free(buffer);
                 lxw_vml_free(vml);
                 return LXW_ERROR_MEMORY_MALLOC_FAILED;
             }
 
             lxw_vml_assemble_xml_file(vml);
 
-            err = _add_file_to_zip(self, vml->file, filename);
+            err = _add_to_zip(self, vml->file, &buffer, &buffer_size,
+                              filename);
 
             fclose(vml->file);
+            free(buffer);
             lxw_vml_free(vml);
 
             RETURN_ON_ERROR(err);
@@ -642,6 +786,8 @@ _write_comment_files(lxw_packager *self)
     lxw_worksheet *worksheet;
     lxw_comment *comment;
     char filename[LXW_FILENAME_LENGTH] = { 0 };
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t index = 1;
     lxw_error err;
 
@@ -661,7 +807,8 @@ _write_comment_files(lxw_packager *self)
         lxw_snprintf(filename, LXW_FILENAME_LENGTH,
                      "xl/comments%d.xml", index++);
 
-        comment->file = lxw_tmpfile(self->tmpdir);
+        comment->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                           self->tmpdir);
         if (!comment->file) {
             lxw_comment_free(comment);
             return LXW_ERROR_CREATING_TMPFILE;
@@ -672,9 +819,11 @@ _write_comment_files(lxw_packager *self)
 
         lxw_comment_assemble_xml_file(comment);
 
-        err = _add_file_to_zip(self, comment->file, filename);
+        err = _add_to_zip(self, comment->file, &buffer, &buffer_size,
+                          filename);
 
         fclose(comment->file);
+        free(buffer);
         lxw_comment_free(comment);
 
         RETURN_ON_ERROR(err);
@@ -690,20 +839,24 @@ STATIC lxw_error
 _write_shared_strings_file(lxw_packager *self)
 {
     lxw_sst *sst = self->workbook->sst;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_error err;
 
     /* Skip the sharedStrings file if there are no shared strings. */
     if (!sst->string_count)
         return LXW_NO_ERROR;
 
-    sst->file = lxw_tmpfile(self->tmpdir);
+    sst->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!sst->file)
         return LXW_ERROR_CREATING_TMPFILE;
 
     lxw_sst_assemble_xml_file(sst);
 
-    err = _add_file_to_zip(self, sst->file, "xl/sharedStrings.xml");
+    err = _add_to_zip(self, sst->file, &buffer, &buffer_size,
+                      "xl/sharedStrings.xml");
     fclose(sst->file);
+    free(buffer);
     RETURN_ON_ERROR(err);
 
     return LXW_NO_ERROR;
@@ -721,6 +874,8 @@ _write_app_file(lxw_packager *self)
     lxw_chartsheet *chartsheet;
     lxw_defined_name *defined_name;
     lxw_app *app;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     uint32_t named_range_count = 0;
     char *autofilter;
     char *has_range;
@@ -733,7 +888,7 @@ _write_app_file(lxw_packager *self)
         goto mem_error;
     }
 
-    app->file = lxw_tmpfile(self->tmpdir);
+    app->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!app->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -791,9 +946,11 @@ _write_app_file(lxw_packager *self)
 
     lxw_app_assemble_xml_file(app);
 
-    err = _add_file_to_zip(self, app->file, "docProps/app.xml");
+    err = _add_to_zip(self, app->file, &buffer, &buffer_size,
+                      "docProps/app.xml");
 
     fclose(app->file);
+    free(buffer);
 
 mem_error:
     lxw_app_free(app);
@@ -809,13 +966,15 @@ _write_core_file(lxw_packager *self)
 {
     lxw_error err = LXW_NO_ERROR;
     lxw_core *core = lxw_core_new();
+    char *buffer = NULL;
+    size_t buffer_size = 0;
 
     if (!core) {
         err = LXW_ERROR_MEMORY_MALLOC_FAILED;
         goto mem_error;
     }
 
-    core->file = lxw_tmpfile(self->tmpdir);
+    core->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!core->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -825,9 +984,11 @@ _write_core_file(lxw_packager *self)
 
     lxw_core_assemble_xml_file(core);
 
-    err = _add_file_to_zip(self, core->file, "docProps/core.xml");
+    err = _add_to_zip(self, core->file, &buffer, &buffer_size,
+                      "docProps/core.xml");
 
     fclose(core->file);
+    free(buffer);
 
 mem_error:
     lxw_core_free(core);
@@ -843,6 +1004,8 @@ _write_metadata_file(lxw_packager *self)
 {
     lxw_error err = LXW_NO_ERROR;
     lxw_metadata *metadata;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
 
     if (!self->workbook->has_metadata)
         return LXW_NO_ERROR;
@@ -854,7 +1017,7 @@ _write_metadata_file(lxw_packager *self)
         goto mem_error;
     }
 
-    metadata->file = lxw_tmpfile(self->tmpdir);
+    metadata->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!metadata->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -862,9 +1025,11 @@ _write_metadata_file(lxw_packager *self)
 
     lxw_metadata_assemble_xml_file(metadata);
 
-    err = _add_file_to_zip(self, metadata->file, "xl/metadata.xml");
+    err = _add_to_zip(self, metadata->file, &buffer, &buffer_size,
+                      "xl/metadata.xml");
 
     fclose(metadata->file);
+    free(buffer);
 
 mem_error:
     lxw_metadata_free(metadata);
@@ -879,6 +1044,8 @@ STATIC lxw_error
 _write_custom_file(lxw_packager *self)
 {
     lxw_custom *custom;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_error err = LXW_NO_ERROR;
 
     if (STAILQ_EMPTY(self->workbook->custom_properties))
@@ -890,7 +1057,7 @@ _write_custom_file(lxw_packager *self)
         goto mem_error;
     }
 
-    custom->file = lxw_tmpfile(self->tmpdir);
+    custom->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!custom->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -900,9 +1067,11 @@ _write_custom_file(lxw_packager *self)
 
     lxw_custom_assemble_xml_file(custom);
 
-    err = _add_file_to_zip(self, custom->file, "docProps/custom.xml");
+    err = _add_to_zip(self, custom->file, &buffer, &buffer_size,
+                      "docProps/custom.xml");
 
     fclose(custom->file);
+    free(buffer);
 
 mem_error:
     lxw_custom_free(custom);
@@ -917,13 +1086,15 @@ _write_theme_file(lxw_packager *self)
 {
     lxw_error err = LXW_NO_ERROR;
     lxw_theme *theme = lxw_theme_new();
+    char *buffer = NULL;
+    size_t buffer_size = 0;
 
     if (!theme) {
         err = LXW_ERROR_MEMORY_MALLOC_FAILED;
         goto mem_error;
     }
 
-    theme->file = lxw_tmpfile(self->tmpdir);
+    theme->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!theme->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -931,9 +1102,11 @@ _write_theme_file(lxw_packager *self)
 
     lxw_theme_assemble_xml_file(theme);
 
-    err = _add_file_to_zip(self, theme->file, "xl/theme/theme1.xml");
+    err = _add_to_zip(self, theme->file, &buffer, &buffer_size,
+                      "xl/theme/theme1.xml");
 
     fclose(theme->file);
+    free(buffer);
 
 mem_error:
     lxw_theme_free(theme);
@@ -948,6 +1121,8 @@ STATIC lxw_error
 _write_styles_file(lxw_packager *self)
 {
     lxw_styles *styles = lxw_styles_new();
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_hash_element *hash_element;
     lxw_error err = LXW_NO_ERROR;
 
@@ -994,7 +1169,7 @@ _write_styles_file(lxw_packager *self)
     styles->dxf_count = self->workbook->used_dxf_formats->unique_count;
     styles->has_comments = self->workbook->has_comments;
 
-    styles->file = lxw_tmpfile(self->tmpdir);
+    styles->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!styles->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -1002,9 +1177,11 @@ _write_styles_file(lxw_packager *self)
 
     lxw_styles_assemble_xml_file(styles);
 
-    err = _add_file_to_zip(self, styles->file, "xl/styles.xml");
+    err = _add_to_zip(self, styles->file, &buffer, &buffer_size,
+                      "xl/styles.xml");
 
     fclose(styles->file);
+    free(buffer);
 
 mem_error:
     lxw_styles_free(styles);
@@ -1019,6 +1196,8 @@ STATIC lxw_error
 _write_content_types_file(lxw_packager *self)
 {
     lxw_content_types *content_types = lxw_content_types_new();
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_workbook *workbook = self->workbook;
     lxw_sheet *sheet;
     char filename[LXW_MAX_ATTRIBUTE_LENGTH] = { 0 };
@@ -1035,7 +1214,8 @@ _write_content_types_file(lxw_packager *self)
         goto mem_error;
     }
 
-    content_types->file = lxw_tmpfile(self->tmpdir);
+    content_types->file = lxw_get_filehandle(&buffer, &buffer_size,
+                                             self->tmpdir);
     if (!content_types->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -1115,9 +1295,11 @@ _write_content_types_file(lxw_packager *self)
 
     lxw_content_types_assemble_xml_file(content_types);
 
-    err = _add_file_to_zip(self, content_types->file, "[Content_Types].xml");
+    err = _add_to_zip(self, content_types->file, &buffer, &buffer_size,
+                      "[Content_Types].xml");
 
     fclose(content_types->file);
+    free(buffer);
 
 mem_error:
     lxw_content_types_free(content_types);
@@ -1132,6 +1314,8 @@ STATIC lxw_error
 _write_workbook_rels_file(lxw_packager *self)
 {
     lxw_relationships *rels = lxw_relationships_new();
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_workbook *workbook = self->workbook;
     lxw_sheet *sheet;
     char sheetname[LXW_FILENAME_LENGTH] = { 0 };
@@ -1144,7 +1328,7 @@ _write_workbook_rels_file(lxw_packager *self)
         goto mem_error;
     }
 
-    rels->file = lxw_tmpfile(self->tmpdir);
+    rels->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!rels->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -1181,9 +1365,11 @@ _write_workbook_rels_file(lxw_packager *self)
 
     lxw_relationships_assemble_xml_file(rels);
 
-    err = _add_file_to_zip(self, rels->file, "xl/_rels/workbook.xml.rels");
+    err = _add_to_zip(self, rels->file, &buffer, &buffer_size,
+                      "xl/_rels/workbook.xml.rels");
 
     fclose(rels->file);
+    free(buffer);
 
 mem_error:
     lxw_free_relationships(rels);
@@ -1199,6 +1385,8 @@ STATIC lxw_error
 _write_worksheet_rels_file(lxw_packager *self)
 {
     lxw_relationships *rels;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_rel_tuple *rel;
     lxw_workbook *workbook = self->workbook;
     lxw_sheet *sheet;
@@ -1226,7 +1414,7 @@ _write_worksheet_rels_file(lxw_packager *self)
 
         rels = lxw_relationships_new();
 
-        rels->file = lxw_tmpfile(self->tmpdir);
+        rels->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
         if (!rels->file) {
             lxw_free_relationships(rels);
             return LXW_ERROR_CREATING_TMPFILE;
@@ -1272,9 +1460,10 @@ _write_worksheet_rels_file(lxw_packager *self)
 
         lxw_relationships_assemble_xml_file(rels);
 
-        err = _add_file_to_zip(self, rels->file, sheetname);
+        err = _add_to_zip(self, rels->file, &buffer, &buffer_size, sheetname);
 
         fclose(rels->file);
+        free(buffer);
         lxw_free_relationships(rels);
 
         RETURN_ON_ERROR(err);
@@ -1291,6 +1480,8 @@ STATIC lxw_error
 _write_chartsheet_rels_file(lxw_packager *self)
 {
     lxw_relationships *rels;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_rel_tuple *rel;
     lxw_workbook *workbook = self->workbook;
     lxw_sheet *sheet;
@@ -1312,7 +1503,7 @@ _write_chartsheet_rels_file(lxw_packager *self)
 
         rels = lxw_relationships_new();
 
-        rels->file = lxw_tmpfile(self->tmpdir);
+        rels->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
         if (!rels->file) {
             lxw_free_relationships(rels);
             return LXW_ERROR_CREATING_TMPFILE;
@@ -1333,9 +1524,10 @@ _write_chartsheet_rels_file(lxw_packager *self)
 
         lxw_relationships_assemble_xml_file(rels);
 
-        err = _add_file_to_zip(self, rels->file, sheetname);
+        err = _add_to_zip(self, rels->file, &buffer, &buffer_size, sheetname);
 
         fclose(rels->file);
+        free(buffer);
         lxw_free_relationships(rels);
 
         RETURN_ON_ERROR(err);
@@ -1352,6 +1544,8 @@ STATIC lxw_error
 _write_drawing_rels_file(lxw_packager *self)
 {
     lxw_relationships *rels;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_rel_tuple *rel;
     lxw_workbook *workbook = self->workbook;
     lxw_sheet *sheet;
@@ -1371,7 +1565,7 @@ _write_drawing_rels_file(lxw_packager *self)
 
         rels = lxw_relationships_new();
 
-        rels->file = lxw_tmpfile(self->tmpdir);
+        rels->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
         if (!rels->file) {
             lxw_free_relationships(rels);
             return LXW_ERROR_CREATING_TMPFILE;
@@ -1388,9 +1582,10 @@ _write_drawing_rels_file(lxw_packager *self)
 
         lxw_relationships_assemble_xml_file(rels);
 
-        err = _add_file_to_zip(self, rels->file, sheetname);
+        err = _add_to_zip(self, rels->file, &buffer, &buffer_size, sheetname);
 
         fclose(rels->file);
+        free(buffer);
         lxw_free_relationships(rels);
 
         RETURN_ON_ERROR(err);
@@ -1408,13 +1603,15 @@ _write_vml_drawing_rels_file(lxw_packager *self, lxw_worksheet *worksheet,
                              uint32_t index)
 {
     lxw_relationships *rels;
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_rel_tuple *rel;
     char sheetname[LXW_FILENAME_LENGTH] = { 0 };
     lxw_error err = LXW_NO_ERROR;
 
     rels = lxw_relationships_new();
 
-    rels->file = lxw_tmpfile(self->tmpdir);
+    rels->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!rels->file) {
         lxw_free_relationships(rels);
         return LXW_ERROR_CREATING_TMPFILE;
@@ -1431,9 +1628,10 @@ _write_vml_drawing_rels_file(lxw_packager *self, lxw_worksheet *worksheet,
 
     lxw_relationships_assemble_xml_file(rels);
 
-    err = _add_file_to_zip(self, rels->file, sheetname);
+    err = _add_to_zip(self, rels->file, &buffer, &buffer_size, sheetname);
 
     fclose(rels->file);
+    free(buffer);
     lxw_free_relationships(rels);
 
     return err;
@@ -1446,6 +1644,8 @@ STATIC lxw_error
 _write_root_rels_file(lxw_packager *self)
 {
     lxw_relationships *rels = lxw_relationships_new();
+    char *buffer = NULL;
+    size_t buffer_size = 0;
     lxw_error err = LXW_NO_ERROR;
 
     if (!rels) {
@@ -1453,7 +1653,7 @@ _write_root_rels_file(lxw_packager *self)
         goto mem_error;
     }
 
-    rels->file = lxw_tmpfile(self->tmpdir);
+    rels->file = lxw_get_filehandle(&buffer, &buffer_size, self->tmpdir);
     if (!rels->file) {
         err = LXW_ERROR_CREATING_TMPFILE;
         goto mem_error;
@@ -1475,9 +1675,10 @@ _write_root_rels_file(lxw_packager *self)
 
     lxw_relationships_assemble_xml_file(rels);
 
-    err = _add_file_to_zip(self, rels->file, "_rels/.rels");
+    err = _add_to_zip(self, rels->file, &buffer, &buffer_size, "_rels/.rels");
 
     fclose(rels->file);
+    free(buffer);
 
 mem_error:
     lxw_free_relationships(rels);
@@ -1519,7 +1720,7 @@ _add_file_to_zip(lxw_packager *self, FILE * file, const char *filename)
     while (size_read) {
 
         if (size_read < self->buffer_size) {
-            if (feof(file) == 0) {
+            if (ferror(file)) {
                 LXW_ERROR("Error reading member file data");
                 RETURN_ON_ZIP_ERROR(error, LXW_ERROR_ZIP_FILE_ADD);
             }
@@ -1546,8 +1747,8 @@ _add_file_to_zip(lxw_packager *self, FILE * file, const char *filename)
 }
 
 STATIC lxw_error
-_add_buffer_to_zip(lxw_packager *self, unsigned char *buffer,
-                   size_t buffer_size, const char *filename)
+_add_buffer_to_zip(lxw_packager *self, char *buffer, size_t buffer_size,
+                   const char *filename)
 {
     int16_t error = ZIP_OK;
 
@@ -1582,6 +1783,17 @@ _add_buffer_to_zip(lxw_packager *self, unsigned char *buffer,
     return LXW_NO_ERROR;
 }
 
+STATIC lxw_error
+_add_to_zip(lxw_packager *self, FILE * file, char **buffer,
+            size_t *buffer_size, const char *filename)
+{
+    /* Flush to ensure buffer is updated when using a memory-backed file. */
+    fflush(file);
+    return *buffer ?
+        _add_buffer_to_zip(self, *buffer, *buffer_size, filename) :
+        _add_file_to_zip(self, file, filename);
+}
+
 /*
  * Write the xml files that make up the XLXS OPC package.
  */
