@@ -1,7 +1,7 @@
 /*
  * Tests for the libxlsxwriter library.
  *
- * Copyright 2014-2021, John McNamara, jmcnamara@cpan.org
+ * Copyright 2014-2022, John McNamara, jmcnamara@cpan.org
  *
  */
 
@@ -641,20 +641,20 @@ CTEST(utility, test_unixtime) {
 
     double got;
 
-    TEST_UNIXTIME(-2209075200,  0);         // 1899-12-31 00:00
-    TEST_UNIXTIME(-2209032000,  0.5);       // 1899-12-31 12:00
-    TEST_UNIXTIME(-2208988800,  1);         // 1900-01-01 00:00
-    TEST_UNIXTIME(-2208945600,  1.5);       // 1900-01-01 12:00
-    TEST_UNIXTIME(-2203977600,  59);        // 1900-02-28 00:00
-    TEST_UNIXTIME(-2203934400,  59.5);      // 1900-02-28 12:00
-    TEST_UNIXTIME(-2203891200,  61);        // 1900-03-01 00:00
-    TEST_UNIXTIME(-2203848000,  61.5);      // 1900-03-01 12:00
-    TEST_UNIXTIME(0,            25569);     // 1970-01-01 00:00
-    TEST_UNIXTIME(43200,        25569.5);   // 1970-01-01 12:00
-    TEST_UNIXTIME(946684800,    36526);     // 2000-01-01 00:00
-    TEST_UNIXTIME(946728000,    36526.5);   // 2000-01-01 12:00
-    TEST_UNIXTIME(253370764800, 2958101);   // 9999-01-01 00:00
-    TEST_UNIXTIME(253370808000, 2958101.5); // 9999-01-01 12:00
+    TEST_UNIXTIME(-2209075200LL,  0);         // 1899-12-31 00:00
+    TEST_UNIXTIME(-2209032000LL,  0.5);       // 1899-12-31 12:00
+    TEST_UNIXTIME(-2208988800LL,  1);         // 1900-01-01 00:00
+    TEST_UNIXTIME(-2208945600LL,  1.5);       // 1900-01-01 12:00
+    TEST_UNIXTIME(-2203977600LL,  59);        // 1900-02-28 00:00
+    TEST_UNIXTIME(-2203934400LL,  59.5);      // 1900-02-28 12:00
+    TEST_UNIXTIME(-2203891200LL,  61);        // 1900-03-01 00:00
+    TEST_UNIXTIME(-2203848000LL,  61.5);      // 1900-03-01 12:00
+    TEST_UNIXTIME(0,              25569);     // 1970-01-01 00:00
+    TEST_UNIXTIME(43200,          25569.5);   // 1970-01-01 12:00
+    TEST_UNIXTIME(946684800,      36526);     // 2000-01-01 00:00
+    TEST_UNIXTIME(946728000,      36526.5);   // 2000-01-01 12:00
+    TEST_UNIXTIME(253370764800,   2958101);   // 9999-01-01 00:00
+    TEST_UNIXTIME(253370808000,   2958101.5); // 9999-01-01 12:00
 
 }
 
